@@ -10,7 +10,6 @@ from mingus.core import diatonic
 import Progressions
 import Options
 import Movements
-import tempfile
 
 APP_NAME = "qtImproviser"
 APP_VERSION = "0.8"
@@ -91,7 +90,7 @@ class ImproviserMainWindow(QtGui.QMainWindow):
 			self.remove_block)
 		self.connect(self.ui.clearblocks,
 			QtCore.SIGNAL("clicked()"),
-			self.ui.blocks.clear)
+			lambda: self.ui.blocks.clear() or self.block_changed())
 		self.connect(self.ui.upblock,
 			QtCore.SIGNAL("clicked()"),
 			lambda: self.swap_list_item(self.ui.blocks))
@@ -176,9 +175,21 @@ class ImproviserMainWindow(QtGui.QMainWindow):
 		self.connect(self.ui.importprogressions,
 			QtCore.SIGNAL("activated()"),
 			self.import_progression)
+		self.connect(self.ui.importinstruments,
+			QtCore.SIGNAL("activated()"),
+			self.import_instruments)
+		self.connect(self.ui.importblocks,
+			QtCore.SIGNAL("activated()"),
+			self.import_blocks)
 		self.connect(self.ui.exportprogressions,
 			QtCore.SIGNAL("activated()"),
 			self.export_progression)
+		self.connect(self.ui.exportinstruments,
+			QtCore.SIGNAL("activated()"),
+			self.export_instruments)
+		self.connect(self.ui.exportblocks,
+			QtCore.SIGNAL("activated()"),
+			self.export_blocks)
 
 		self.connect(self.ui.movement,
 			QtCore.SIGNAL("highlighted(int)"),
@@ -191,6 +202,9 @@ class ImproviserMainWindow(QtGui.QMainWindow):
 			self.connect(self.ui.uploadsong,
 				QtCore.SIGNAL("activated()"),
 				self.upload_song)
+			self.connect(self.ui.uploadprogressions,
+				QtCore.SIGNAL("activated()"),
+				self.upload_progression)
 
 			self.ui.uploadbutton.setEnabled(True)
 			self.ui.menuUpload.setEnabled(True)
@@ -206,6 +220,12 @@ class ImproviserMainWindow(QtGui.QMainWindow):
 
 	def upload_song(self):
 
+		if self.ui.progressions.count() == 0:
+			q = QtGui.QMessageBox.warning(self, "Error", "No progressions selected.", 1, 0)
+			return
+		if self.ui.instruments.count() == 0:
+			q = QtGui.QMessageBox.warning(self, "Error", "No instruments selected.", 1, 0)
+			return
 		u = UploadDialog(
 			Options.UPLOAD_SONG,
 			Options.get_song_as_text(self.get_options()),
@@ -215,6 +235,21 @@ class ImproviserMainWindow(QtGui.QMainWindow):
 		u.show()
 		u.exec_()
 
+	def upload_progression(self):
+		if self.ui.progressions.count() == 0:
+			q = QtGui.QMessageBox.warning(self, "Error", "No progressions to upload.", 1, 0)
+			return
+		u = UploadDialog(
+			Options.UPLOAD_PROGRESSION,
+			self.get_progressions(),
+			self.credentials)
+		u.setWindowTitle("Upload Progression") 
+		u.ui.uploadname.setText(self.ui.projectname.text())
+		u.show()
+		u.exec_()
+
+
+
 	def set_defaults(self):
 
 		self.lastfile = ""
@@ -249,21 +284,25 @@ class ImproviserMainWindow(QtGui.QMainWindow):
 		self.set_title()
 		self.progression_changed()
 		self.instrument_changed()
+		self.block_changed()
 	
 	def add_block(self):
 		self.ui.blocks.addItem("Block")
 		self.ui.blocks.setCurrentRow(int(self.ui.blocks.count()) - 1)
 		self.edit_block()
+		self.block_changed()
 
 	def remove_block(self):
 		i = self.ui.blocks.currentRow()
 		if i >= 0:
 			self.ui.blocks.takeItem(i)
+		self.block_changed()
 
 	def copy_block(self):
 		i = self.ui.blocks.currentRow()
 		if i >= 0:
 			self.ui.blocks.addItem(self.ui.blocks.item(i).text())
+		self.block_changed()
 
 	def add_progression(self):
 		self.ui.progressions.addItem(
@@ -308,6 +347,56 @@ class ImproviserMainWindow(QtGui.QMainWindow):
 		self.progression_changed()
 		self.ui.Tabs.setCurrentIndex(4)
 
+	def import_instruments(self):
+		h = "/home"
+		if 'HOME' in environ:
+			h = environ["HOME"]
+		f = QtGui.QFileDialog()
+		s = f.getOpenFileName(self, "Load instruments.",
+				h, "Instruments (*.ins)")
+		if s != "":
+			try:
+				fp = open(s, "r")
+				for x in fp.readlines():
+					instr = x.split(",")
+					for i in instr:
+						try:
+							params = i.split()
+							if params[0] in Options.get_available_instruments():
+								self.ui.instruments.addItem(i)
+						except:
+							pass
+				fp.close()
+			except Options.OptionError, err:
+				self.set_status("Error: " + str(err))
+		self.instrument_changed()
+		self.ui.Tabs.setCurrentIndex(3)
+
+	def import_blocks(self):
+		h = "/home"
+		if 'HOME' in environ:
+			h = environ["HOME"]
+		f = QtGui.QFileDialog()
+		s = f.getOpenFileName(self, "Load blocks.",
+				h, "Blocks (*.blk)")
+		if s != "":
+			try:
+				fp = open(s, "r")
+				for x in fp.readlines():
+					blocks = x.split(",")
+					for b in blocks:
+						try:
+							params = b.split()
+							if params[0] in Options.get_available_blocks():
+								self.ui.blocks.addItem(b)
+						except:
+							pass
+				fp.close()
+			except Options.OptionError, err:
+				self.set_status("Error: " + str(err))
+		self.block_changed()
+		self.ui.Tabs.setCurrentIndex(2)
+
 
 	def export_progression(self):
 		h = "/home"
@@ -331,7 +420,54 @@ class ImproviserMainWindow(QtGui.QMainWindow):
 				fp.close()
 			except IOError:
 				self.set_status("Error: Couldn't open file for writing")
-		self.progression_changed()
+
+	def export_instruments(self):
+		h = "/home"
+		if 'HOME' in environ:
+			h = environ["HOME"]
+		instr = self.get_instruments()
+		if instr == "":
+			return 
+		
+		f = QtGui.QFileDialog()
+		s = str(f.getSaveFileName(self, "Save instruments.",
+				h, "Instruments (*.ins)"))
+		if s != "":
+			if s[-4:].lower() != ".ins":
+				filename = s + ".ins"
+			else:
+				filename = s
+			try:
+				fp = open(filename, "w")
+				fp.write(instr)
+				fp.close()
+			except IOError:
+				self.set_status("Error: Couldn't open file for writing")
+
+	def export_blocks(self):
+		h = "/home"
+		if 'HOME' in environ:
+			h = environ["HOME"]
+		blocks = self.get_blocks()
+		if blocks == "":
+			return 
+		
+		f = QtGui.QFileDialog()
+		s = str(f.getSaveFileName(self, "Save blocks.",
+				h, "Blocks (*.blk)"))
+		if s != "":
+			if s[-4:].lower() != ".blk":
+				filename = s + ".blk"
+			else:
+				filename = s
+			try:
+				fp = open(filename, "w")
+				fp.write(blocks)
+				fp.close()
+			except IOError:
+				self.set_status("Error: Couldn't open file for writing")
+
+
 
 	def progression_changed(self):
 		if self.ui.progressions.count() == 0:
@@ -345,9 +481,15 @@ class ImproviserMainWindow(QtGui.QMainWindow):
 		else:
 			self.enable_instruments(True)
 		
+	def block_changed(self):
+		if self.ui.blocks.count() == 0:
+			self.enable_blocks(False)
+		else:
+			self.enable_blocks(True)
 
 	def enable_progression(self, enable = True):
 		self.ui.exportprogressions.setEnabled(enable)
+		self.ui.uploadprogressions.setEnabled(enable)
 		self.ui.upprogression.setEnabled(False)
 		self.ui.downprogression.setEnabled(False)
 		self.ui.clearprogressions.setEnabled(enable)
@@ -360,6 +502,8 @@ class ImproviserMainWindow(QtGui.QMainWindow):
 			self.ui.downprogression.setEnabled(enable)
 
 	def enable_instruments(self, enable = True):
+		self.ui.exportinstruments.setEnabled(enable)
+		self.ui.uploadinstruments.setEnabled(enable)
 		self.ui.upinstrument.setEnabled(False)
 		self.ui.downinstrument.setEnabled(False)
 		self.ui.clearinstruments.setEnabled(enable)
@@ -371,6 +515,21 @@ class ImproviserMainWindow(QtGui.QMainWindow):
 			self.ui.upinstrument.setEnabled(enable)
 			self.ui.downinstrument.setEnabled(enable)
 
+	def enable_blocks(self, enable = True):
+		self.ui.exportblocks.setEnabled(enable)
+		self.ui.uploadblocks.setEnabled(enable)
+		self.ui.upblock.setEnabled(False)
+		self.ui.downblock.setEnabled(False)
+		self.ui.clearblocks.setEnabled(enable)
+		self.ui.copyblock.setEnabled(enable)
+		self.ui.editblock.setEnabled(enable)
+		self.ui.removeblock.setEnabled(enable)
+
+		if enable and self.ui.blocks.count() > 1:
+			self.ui.upblock.setEnabled(enable)
+			self.ui.downblock.setEnabled(enable)
+		
+
 	def add_instrument(self):
 		self.ui.instruments.addItem(self.ui.instrumentcombo.currentText())
 		self.ui.instruments.setCurrentRow(int(self.ui.instruments.count()) -1)
@@ -398,6 +557,7 @@ class ImproviserMainWindow(QtGui.QMainWindow):
 		b.load_block(str(cur.text()))
 		b.show()
 		b.exec_()
+		self.block_changed()
 
 	def edit_instrument(self):
 		cur = self.ui.instruments.item(self.ui.instruments.currentRow())
@@ -561,6 +721,7 @@ class ImproviserMainWindow(QtGui.QMainWindow):
 					pass
 		self.progression_changed()
 		self.instrument_changed()
+		self.block_changed()
 
 	def get_options(self):
 		o = OptionClass()
